/* ------------------------------------------------------------------------
   (C) Copyright Marxmeier Software AG 2009-2014

   Project:  Eloquence JDLG Plugin Interface
   File:     Datepicker.java
   Function: JDLG Plugin: Datepicker
   $Id: Datepicker.java,v 1.14 2014/12/11 12:15:42 rhg Exp $

   Edit history:
   2009-04-06, ap   - creation
   2009-09-25, rhg  - finalized
   2014-04-08, rhg  - api version 1.1 additions
   2014-12-04, rhg  - scale MonthView font
 ------------------------------------------------------------------------ */

package com.eloquence.plugin;

import com.eloquence.api.*;

import java.awt.*;
import java.text.*;
import java.util.*;
import java.awt.event.*;

import org.jdesktop.swingx.*;

public class Datepicker
   extends JXDatePicker
   implements Plugin, ActionListener
{
   private Jdlg jdlg;

   // ------------------------------------------------------------------------
   // Plugin
   // ------------------------------------------------------------------------

   public Component onCreate(Jdlg jdlg)
   {
      if (!jdlg.requiresApiVersion(1, 0))
         return null;

      this.jdlg = jdlg;
      addActionListener(this);
      return this;
   }

   public void onDispose()
   {
      removeActionListener(this);
   }

   public void onEloq2Swing()
   {
      if (dateModified)
      {
         Date d = null;
         try
         {
            if (date.length() > 0)
            {
               if (dateFormat == null)
                  dateFormat = new SimpleDateFormat(format);
               d = dateFormat.parse(date);
            }
         }
         catch (Throwable t)
         {
         }

         if (d == null)
         {
            Calendar calendar = Calendar.getInstance();
            d = calendar.getTime();
         }

         setDate(d);
         dateModified = false;
      }
   }

   public void onSwing2Eloq()
   {
      if (dateFormat == null)
         dateFormat = new SimpleDateFormat(format);
      date = dateFormat.format(getDate());
   }

   public void onFontModified(Font font)
   {
      setFont(font);

      JXMonthView monthView = getMonthView();
      Font currentViewFont = monthView.getFont();
      Font newViewFont = new Font(currentViewFont.getFontName(),
                                  currentViewFont.getStyle(), font.getSize());
      monthView.setFont(newViewFont);
   }

   public boolean isFocusable()
   {
      return true;
   }

   public void onFocusReceived()
   {
   }

   public void onFocusLost()
   {
   }

   public Component onGetFocusPeer()
   {
      return null;
   }

   public void onTabFocus(boolean toNext)
   {
   }

   public boolean fireRuleOnLosingFocus()
   {
      return false;
   }

   public boolean onRuleOverride()
   {
      return true;
   }

   public boolean allowRMBR()
   {
      return true;
   }

   // ------------------------------------------------------------------------
   // ActionListener
   // ------------------------------------------------------------------------

   public void actionPerformed(ActionEvent event)
   {
      jdlg.triggerRule();
   }

   // ------------------------------------------------------------------------
   // Events
   // ------------------------------------------------------------------------

   public void processEvent(AWTEvent event)
   {
      if (!jdlg.ignoreEvent(event))
         super.processEvent(event);
   }

   // ------------------------------------------------------------------------
   // get/set date
   // ------------------------------------------------------------------------

   private static final String DATE_ATTR_NAME = "date";
   private String date = "";
   private boolean dateModified = true;

   private final void setDateAttr(String date)
   {
      this.date = date;
      dateModified = true;
   }

   private final String getDateAttr()
   {
      return date;
   }

   // ------------------------------------------------------------------------
   // get/set format
   // ------------------------------------------------------------------------

   private static final String FORMAT_ATTR_NAME = "format";
   private String format = "dd.MM.yyyy";
   SimpleDateFormat dateFormat = null;

   private final void setFormat(String format)
   {
      this.format = format;
      dateFormat = null;
      dateModified = true;
   }

   private final String getFormat()
   {
      return format;
   }

   // ------------------------------------------------------------------------
   // attributes
   // ------------------------------------------------------------------------

   private static final int DATE_ATTR_ID = 1;
   private static final int FORMAT_ATTR_ID = 2;

   public void onClone(Plugin clone)
   {
      Datepicker cloneObj = (Datepicker)clone;
      cloneObj.setDateAttr(getDateAttr());
      cloneObj.setFormat(getFormat());
   }

   public boolean onSet(String attribute, java.util.List index,
                        UnifiedData data)
   {
      Integer attrInt = (Integer)attributeList.get(attribute);
      if (attrInt != null)
      {
         int indexElements = index != null ? index.size() : 0;
         switch (attrInt.intValue())
         {
            case DATE_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (!data.isStringData())
                  return false;
               setDateAttr(data.getStringData());
               break;

            case FORMAT_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (!data.isStringData())
                  return false;
               setFormat(data.getStringData());
               break;
         }

         return true;
      }

      return false;
   }

   public UnifiedData onGet(String attribute, java.util.List index)
   {
      Integer attrInt = (Integer)attributeList.get(attribute);
      if (attrInt != null)
      {
         int indexElements = index != null ? index.size() : 0;
         switch (attrInt.intValue())
         {
            case DATE_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getDateAttr());

            case FORMAT_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getFormat());
         }
      }

      return null;
   }

   private static HashMap attributeList = new HashMap();
   static
   {
      attributeList.put(DATE_ATTR_NAME, new Integer(DATE_ATTR_ID));
      attributeList.put(FORMAT_ATTR_NAME, new Integer(FORMAT_ATTR_ID));
   }
}
